// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

//
//  W A R N I N G
//  -------------
//
// This file is not part of the QtDataVisualization API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.

#ifndef QBARDATAITEM_P_H
#define QBARDATAITEM_P_H

#include "datavisualizationglobal_p.h"
#include "qbardataitem.h"

QT_BEGIN_NAMESPACE

class QBarDataItemPrivate
{
public:
    QBarDataItemPrivate();
    virtual ~QBarDataItemPrivate();
};

QT_END_NAMESPACE

#endif
